@@ -16,9 +16,10 @@
  */
 
 SEXP R_apply_dist_matrix(SEXP p) {
-    int i, j, k, l, n, nx, ny, nz, m = 0;
+    int i, j, k, n, nx, ny, nz, m = 0;
     SEXP r, c, tx, ty;
     SEXP R_x, x, R_y, y, R_d, f;
+    R_xlen_t l;
 
     p = CDR(p);
     if (length(p) < 4)
@@ -64,8 +65,9 @@ SEXP R_apply_dist_matrix(SEXP p) {
 
     if (m == 0) {
 	SEXP d;
-	
-        PROTECT(r = allocVector(REALSXP, nx*(nx-1)/2));
+
+	l = nx;	
+        PROTECT(r = allocVector(REALSXP, l*(l-1)/2));
 	setAttrib(r, install("Size"), PROTECT(ScalarInteger(nx)));
 	UNPROTECT(1);
 	
@@ -144,9 +146,10 @@ SEXP R_apply_dist_matrix(SEXP p) {
  */
 
 SEXP R_apply_dist_list(SEXP p) {
-    int i, j, l, nx, ny, nz, m = 0;
+    int i, j, nx, ny, nz, m = 0;
     SEXP r, c, d, tx, ty;
     SEXP x, y, f;
+    R_xlen_t l;
 
     p = CDR(p);
     if (length(p) < 4)
@@ -176,14 +179,17 @@ SEXP R_apply_dist_list(SEXP p) {
 
     if (m == 0) {
 	SEXP d;
-	
-        PROTECT(r = allocVector(REALSXP, nx*(nx-1)/2));
+
+	l = nx;	
+        PROTECT(r = allocVector(REALSXP, l*(l-1)/2));
 	
 	setAttrib(r, install("Size"), PROTECT(ScalarInteger(nx)));
 	UNPROTECT(1);
 	
-	if (!isNull(d = getAttrib(x, R_NamesSymbol)))
+	PROTECT(d = getAttrib(x, R_NamesSymbol));
+	if (!isNull(d))
 	    setAttrib(r, install("Labels"), d);
+	UNPROTECT(1);
 	// fixme: package?
 	setAttrib(r, R_ClassSymbol, PROTECT(mkString("dist")));
 	UNPROTECT(1);
@@ -193,8 +199,8 @@ SEXP R_apply_dist_list(SEXP p) {
 	
         PROTECT(r = allocMatrix(REALSXP, nx, ny));
 	
-	d1 = getAttrib(x, R_NamesSymbol);
-	d2 = getAttrib(y, R_NamesSymbol);
+	PROTECT(d1 = getAttrib(x, R_NamesSymbol));
+	PROTECT(d2 = getAttrib(y, R_NamesSymbol));
 	if (!isNull(d1) || !isNull(d2)) {
 	    SEXP d;
 
@@ -203,15 +209,17 @@ SEXP R_apply_dist_list(SEXP p) {
 	    SET_VECTOR_ELT(d, 0, d1);
 	    SET_VECTOR_ELT(d, 1, d2);
 	}
+	UNPROTECT(2);
     } else {
 	if (nx != ny)
 	    error("the number of components of 'x' and 'y' does not conform");
 
 	PROTECT(r = allocVector(REALSXP, nx));
     }
-    
-    PROTECT(c = LCONS(f, (tx = CONS(R_NilValue, 
-                         (ty = CONS(R_NilValue, p))))));
+
+    PROTECT(ty = CONS(R_NilValue, p));
+    PROTECT(tx = CONS(R_NilValue, ty));
+    PROTECT(c = LCONS(f, tx));
    
     l = 0;
     for (j = 0; j < ny; j++) {
@@ -241,7 +249,7 @@ SEXP R_apply_dist_list(SEXP p) {
         R_CheckUserInterrupt();
     }
 
-    UNPROTECT(2);
+    UNPROTECT(4);
 
     return r;
 }
@@ -254,10 +262,11 @@ SEXP R_apply_dist_list(SEXP p) {
  */
 
 SEXP R_apply_dist_binary_matrix(SEXP p) {
-    int i, j, k, l, n, nx, ny, nz, m = 0;   
+    int i, j, k, n, nx, ny, nz, m = 0;   
     int i0, j0;
     SEXP r, c, d, ta, tb, tc, td, tn;
     SEXP x, y, f;
+    R_xlen_t l;
 
     p = CDR(p);
     if (length(p) < 3)
@@ -291,8 +300,9 @@ SEXP R_apply_dist_binary_matrix(SEXP p) {
 
     if (m == 0) {
 	SEXP d;
-	
-        PROTECT(r = allocVector(REALSXP, nx*(nx-1)/2));
+
+	l = nx;	
+        PROTECT(r = allocVector(REALSXP, l*(l-1)/2));
 	
 	setAttrib(r, install("Size"), PROTECT(ScalarInteger(nx)));
 	UNPROTECT(1);
@@ -414,9 +424,10 @@ static void setElement(SEXP x, int i, SEXP y) {
 }
 
 SEXP R_apply_dist_data_frame(SEXP p) {
-    int i, j, k, l, nc, nx, ny, nz, m = 0;
+    int i, j, k, nc, nx, ny, nz, m = 0;
     SEXP r, c, d, tx, ty, rx, ry;
     SEXP x, y, f;
+    R_xlen_t l;
 
     p = CDR(p);
     if (length(p) < 4)
@@ -471,7 +482,8 @@ SEXP R_apply_dist_data_frame(SEXP p) {
 
     // fixme: row.names
     if (m == 0) {
-        PROTECT(r = allocVector(REALSXP, nx*(nx-1)/2));
+	l = nx;
+        PROTECT(r = allocVector(REALSXP, l*(l-1)/2));
 
 	setAttrib(r, install("Size"), PROTECT(ScalarInteger(nx)));
 	UNPROTECT(1);
@@ -514,12 +526,10 @@ SEXP R_apply_dist_data_frame(SEXP p) {
 
 					// fixme: should fail for S4
 	SET_VECTOR_ELT(tx, k, (s = allocVector(TYPEOF(t), 1)));
-	SET_ATTRIB(s, ATTRIB(t));	// fixme: may be wrong
-	SET_OBJECT(s, OBJECT(t));
+	DUPLICATE_ATTRIB(s, ATTRIB(t));	// fixme: may be wrong
 
 	SET_VECTOR_ELT(ty, k, (s = allocVector(TYPEOF(t), 1)));
-	SET_ATTRIB(s, ATTRIB(t));
-	SET_OBJECT(s, OBJECT(t));
+	DUPLICATE_ATTRIB(s, ATTRIB(t));
     }
 
     PROTECT(c = LCONS(f, CONS(tx, CONS(ty, p))));
